#!/usr/bin/env python3
# -*- coding: utf-8; -*-




from katoptron.eigenvalues import *
import os
import fwk
from scipy import sparse


def write_mm(filename, matrix, drow=0, dcol=0):
    f = open(filename, 'w')
    print('%%MatrixMarket matrix', file=f)
    print(str(int(matrix.shape[0])) + ' ' + str(int(matrix.shape[1])) + ' ' + str(int(matrix.count_nonzero())), file=f)

    row, col = matrix.nonzero()
    for i in range(0, matrix.count_nonzero()):
        print(str(int(row[i]+1+drow)) + ' ' + str(int(col[i]+1+dcol)) + ' ' + '{0:1.17e}'.format(matrix[row[i], col[i]]), file=f)

    f.closed


def write_vector_mm(filename, vec, n):
    f = open(filename, 'w')
    print('%%MatrixMarket matrix', file=f)
    print(str(int(n)), file=f)
    for i in range(0, n):
        if i < vec.size:
            print('{0:1.17e}'.format(vec[i]), file=f)
        else:
            print('{0:1.17e}'.format(0.), file=f)

    f.closed


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    input_file_dir = '/Users/kimliegeois/Desktop/workspace 5/katoptron_tests_beam'

    filename_Q = input_file_dir + '/ApCTC_mm.txt'
    filename_GT = input_file_dir + '/B_mm.txt'
    filename_f = input_file_dir + '/b_mm_after_bc.txt'

    Q = read_mm(filename_Q)
    Q = get_symmetrical_part(Q)
    GT = read_mm(filename_GT)
    G = GT.transpose()
    f = read_mm(filename_f, is_sparse=False)

    print(Q.shape)
    print(Q.count_nonzero())

    write_vector_mm('f_mm.txt', f, G.shape[0]+G.shape[1])

    write_mm('Q_mm.txt', Q)
    write_mm('G_mm.txt', G, dcol=Q.shape[0])
    write_mm('GT_mm.txt', GT, drow=Q.shape[0])

    Afull = read_mm(
        "/Users/kimliegeois/Desktop/workspace 7/katoptron_tests_beam/Applied_Matrix_mm.txt", is_sparse=False)
    A = sparse.csr_matrix(Afull)

    print(A.count_nonzero())

    write_mm('A_full_mm.txt', A)
    write_mm('C_mm.txt', A[Q.shape[0]:A.shape[0], Q.shape[1]:A.shape[1]])


if __name__ == "__main__":
    main()
