#!/usr/bin/env python3
# -*- coding: utf-8; -*-

from fwk.testing import *
import matplotlib.tri as tri
from mpl_toolkits.mplot3d import Axes3D
from matplotlib import cm
import matplotlib.pyplot as plt
import katoptron.utilities as u
import numpy as np
import os
import matplotlib
matplotlib.use('TkAgg')  # <-- THIS MAKES IT FAST!


def rigid_body_mode(dz, theta_x, theta_y, x, y, x_c, y_c):
    dz_test = np.zeros(x.shape)
    dz_test = -np.sin(theta_x)*(y-y_c)+np.cos(theta_x)*dz_test
    dz_test = -np.sin(theta_y)*(x-x_c)+np.cos(theta_y)*dz_test
    dz_test = dz_test + dz
    return dz_test


def main():

    file_dir = os.path.dirname(__file__)

    npzfile = np.load(file_dir+'/mirror_data.npz')
    x = npzfile['x']
    y = npzfile['y']
    z = np.zeros(x.shape)
    tri = npzfile['tri']
    dz = npzfile['dz']

    dz_1 = 0
    dz_2 = 0.5/1000
    dz_3 = 0.87/1000
    dz_4 = (-2. / 1000)

    theta_x_1 = 0
    theta_x_2 = 0.0015
    theta_x_3 = -0.0005
    theta_x_4 = -0.0002

    theta_y_1 = -0.2
    theta_y_2 = -0.0005
    theta_y_3 = 0.0015
    theta_y_4 = 0.0008

    passed_1 = 0
    passed_2 = 1
    passed_3 = 1
    passed_4 = 0

    dz_test_1 = rigid_body_mode(dz_1, theta_x_1, theta_y_1, x, y, 0.069, 0.04)
    dz_test_2 = rigid_body_mode(dz_2, theta_x_2, theta_y_2, x, y, 0.069, 0.04)
    dz_test_3 = rigid_body_mode(dz_3, theta_x_3, theta_y_3, x, y, 0.069, 0.04)
    dz_test_4 = rigid_body_mode(dz_4, theta_x_4, theta_y_4, x, y, 0.069, 0.04)

    '''
    levels = np.linspace(min(dz), max(dz), 15)
    # Velocity x-component
    fig = plt.figure(figsize=(8, 8))
    plt.set_cmap('jet')
    plt.tricontour(x, y, tri, dz, levels, linestyles='-',
                colors='black', linewidths=0.5)
    axtri = plt.tricontourf(x, y, tri, dz, levels, cmap='coolwarm')
    plt.plot(0.069,0.04,'ko')
    plt.gca().set_aspect('equal')
    
    fig = plt.figure(figsize=(16,9))
    ax = fig.gca(projection='3d', proj_type = 'ortho')
    ax.plot_trisurf(x,y,dz_test_1,triangles=tri, cmap='coolwarm')
    '''
    # plt.show()

    optical_coefficients_1, dz_residual_1 = u.compute_RBM_Curvature_Irreg(
        x, y, z, tri, dz_test_1, "m", np.array([0.069, 0.04]))

    optical_coefficients_2, dz_residual_2 = u.compute_RBM_Curvature_Irreg(
        x, y, z, tri, dz_test_2, "m", np.array([0.069, 0.04]))

    optical_coefficients_3, dz_residual_3 = u.compute_RBM_Curvature_Irreg(
        x, y, z, tri, dz_test_3, "m", np.array([0.069, 0.04]))

    optical_coefficients_4, dz_residual_4 = u.compute_RBM_Curvature_Irreg(
        x, y, z, tri, dz_test_4, "m", np.array([0.069, 0.04]))

    tests = CTests()
    measured_dz_1 = optical_coefficients_1[0]
    tests.add(CTest('piston mode test 1', measured_dz_1, dz_1*1000, 0.05))
    measured_dz_2 = optical_coefficients_2[0]
    tests.add(CTest('piston mode test 2', measured_dz_2, dz_2*1000, 0.05))
    measured_dz_3 = optical_coefficients_3[0]
    tests.add(CTest('piston mode test 3', measured_dz_3, dz_3*1000, 0.05))
    measured_dz_4 = optical_coefficients_4[0]
    tests.add(CTest('piston mode test 4', measured_dz_4, dz_4*1000, 0.05))

    measured_ry_1 = optical_coefficients_1[1]
    tests.add(CTest('y rotation test 1', measured_ry_1, theta_y_1, 0.05))
    measured_ry_2 = optical_coefficients_2[1]
    tests.add(CTest('y rotation test 2', measured_ry_2, theta_y_2, 0.05))
    measured_ry_3 = optical_coefficients_3[1]
    tests.add(CTest('y rotation test 3', measured_ry_3, theta_y_3, 0.05))
    measured_ry_4 = optical_coefficients_4[1]
    tests.add(CTest('y rotation test 4', measured_ry_4, theta_y_4, 0.05))

    measured_rx_1 = optical_coefficients_1[2]
    tests.add(CTest('x rotation test 1', measured_rx_1, theta_x_1, 0.05))
    measured_rx_2 = optical_coefficients_2[2]
    tests.add(CTest('x rotation test 2', measured_rx_2, theta_x_2, 0.05))
    measured_rx_3 = optical_coefficients_3[2]
    tests.add(CTest('x rotation test 3', measured_rx_3, theta_x_3, 0.05))
    measured_rx_4 = optical_coefficients_4[2]
    tests.add(CTest('x rotation test 4', measured_rx_4, theta_x_4, 0.05))

    measured_cur_1 = optical_coefficients_1[3]
    tests.add(CTest('curvature change test 1', measured_cur_1, 0., 1e-10))
    measured_cur_2 = optical_coefficients_2[3]
    tests.add(CTest('curvature change test 2', measured_cur_2, 0., 1e-10))
    measured_cur_3 = optical_coefficients_3[3]
    tests.add(CTest('curvature change test 3', measured_cur_3, 0., 1e-10))
    measured_cur_4 = optical_coefficients_4[3]
    tests.add(CTest('curvature change test 4', measured_cur_4, 0., 1e-10))

    measured_irr_1 = optical_coefficients_1[4]
    tests.add(CTest('irregularity change test 1', measured_irr_1, 0., 1e-10))
    measured_irr_2 = optical_coefficients_2[4]
    tests.add(CTest('irregularity change test 2', measured_irr_2, 0., 1e-10))
    measured_irr_3 = optical_coefficients_3[4]
    tests.add(CTest('irregularity change test 3', measured_irr_3, 0., 1e-10))
    measured_irr_4 = optical_coefficients_4[4]
    tests.add(CTest('irregularity change test 4', measured_irr_4, 0., 1e-10))

    blurred_1 = u.check_optical_tolerance(optical_coefficients_1, 'mm')
    blurred_2 = u.check_optical_tolerance(optical_coefficients_2, 'mm')
    blurred_3 = u.check_optical_tolerance(optical_coefficients_3, 'mm')
    blurred_4 = u.check_optical_tolerance(optical_coefficients_4, 'mm')
    tests.add(CTest('checl optical tolerance test 1', blurred_1, passed_1))
    tests.add(CTest('checl optical tolerance test 2', blurred_2, passed_2))
    tests.add(CTest('checl optical tolerance test 3', blurred_3, passed_3))
    tests.add(CTest('checl optical tolerance test 4', blurred_4, passed_4))

    tests.run()


if __name__ == "__main__":
    main()
