#ifndef KATOPTRON_ELEMENTCOMPUTATION_H
#define KATOPTRON_ELEMENTCOMPUTATION_H

#include "Stokhos_Sacado_Kokkos_MP_Vector.hpp"

#include <Kokkos_Core.hpp>
#include "Kokkos_ViewFactory.hpp"
#include "wCacheHex8.h"
#include "wCacheQuad4.h"
#include "wCacheTetra4.h"
#include "wCacheTri3.h"
#include "tMatrix.h"
#include "Sacado.hpp"
#include "Domain.h"

#include "MaterialsList.h"
#include "Material.h"

#include "Map.h"

#include "wElement.h"

#include "wLazy.h"
#include "tMatrix.h"

/**
 * @brief Base class for the element computations
 */
template <typename scalar, int element_type>
class ElementComputation
{
    typedef katoptron::Map::local_ordinal_type local_ordinal_type;

protected:
    tbox::CacheHex8 &trilinosHex8GetCache();
    tbox::CacheTetra4 &trilinosTetra4GetCache();
    tbox::CacheTri3 &trilinosTri3GetCache();
    tbox::CacheQuad4 &trilinosQuad4GetCache();

    double buildJ(int k, tMatrix<double, 3, 3> &J,
                  local_ordinal_type e,
                  const katoptron::ElementsList &elementsList,
                  const katoptron::NodesList &nodesList);
};

#endif //KATOPTRON_ELEMENTCOMPUTATION_H
